const express = require('express');
const pg = require('pg');
const bcrypt = require('bcrypt');
const cors = require('cors');
const fs = require('fs');
const path = require('path');

const app = express();
const port = 21; // You can change the port if needed

// PostgreSQL setup
const pool = new pg.Pool({
  user: 'postgres',
  host: 'containers-us-west-209.railway.app',
  database: 'postgresql://postgres:N5qUZruZWCzRwwwqmD3z@containers-us-west-209.railway.app:6469/railway',
  password: 'N5qUZruZWCzRwwwqmD3z',
  port: 6469, // Default PostgreSQL port
});

app.use(express.json()); // Replace bodyParser with express.json()
app.use(cors());

// Generate a 4-digit OTP
function generateOTP() {
    return Math.floor(1000 + Math.random() * 9000);
}
// Signup Endpoint
app.post('/signup', async (req, res) => {
  const { email, password, confirmPassword } = req.body;
  console.log('Request body:', req.body); 
  // Check if the password and confirm password match
  if (password !== confirmPassword) {
    return res.status(400).json({ code: 400, status: 'error', error: 'Passwords do not match' });
  }

  try {
    // Hash the password
    console.log('Password before hashing:', password); // Check password value

    // Hash the password
    const hashedPassword = await bcrypt.hash(password, 10);

    console.log('Hashed password:', hashedPassword); // Check hashed password value

    // Generate a 4-digit OTP (Assuming generateOTP() function is implemented)
    const otp = generateOTP();

    console.log('Generated OTP:', otp); 

    // Insert the new user data into the database
    const insertResult = await insertUserIntoDatabase(email, hashedPassword, otp);
    const userId = insertResult.rows[0].id;

    // Send OTP to the user's email (Implement this part)

    // Respond with the desired format
    res.status(200).json({
      code: 200,
      status: 'success',
      data: {
        usercustomerId: userId,
        email: email,
        otp: otp,
      },
    });
  } catch (error) {
    console.error('Error while signing up:', error);
    res.status(500).json({ code: 500, status: 'error', error: 'Internal Server Error' });
  }
});

// Define a separate function to insert the user into the database
// You can also use an ORM library like Sequelize for database interactions
async function insertUserIntoDatabase(email, hashedPassword, otp) {
  const client = await pool.connect();

  try {
    // Check if the email already exists in the database
    const emailExistsResult = await client.query('SELECT * FROM users WHERE email = $1', [email]);

    if (emailExistsResult.rows.length > 0) {
      // Email already exists
      throw new Error('Email already exists');
    }

    // Insert the new user data into the database
    const insertResult = await client.query(
      'INSERT INTO users (email, password, otp) VALUES ($1, $2, $3) RETURNING id',
      [email, hashedPassword, otp]
    );

    return insertResult;
  } finally {
    client.release();
  }
}

app.post('/verify-otp', async (req, res) => {
    const { usercustomerId, otp } = req.body;
     console.log('Received usercustomerId:', usercustomerId);

  
    try {
      const client = await pool.connect();
  
      // Fetch the user data from the database based on the provided usercustomerId
      const userQueryResult = await client.query('SELECT * FROM users WHERE id = $1', [usercustomerId]);
  
      if (userQueryResult.rows.length === 0) {
        // User not found, return an error response
        client.release();
        return res.status(404).json({ code: 404, status: 'error', error: 'User not found' });
      }
  
      const user = userQueryResult.rows[0];
  
      // Check if the provided OTP matches the OTP in the database
      if (user.otp !== otp) {
        // Incorrect OTP, return an error response
        client.release();
        return res.status(401).json({ code: 401, status: 'error', error: 'Incorrect OTP' });
      }
  
      // OTP matched, return the success response with user data
      client.release();
      res.status(200).json({
        code: 200,
        status: 'success',
        data: {
          usercustomerId: user.id,
          email: user.email,
          // You can include other user data here if needed
        },
      });
    } catch (error) {
      res.status(500).json({ code: 500, status: 'error', error: 'Internal Server Error' });
    }
});

// Function to convert an image to Base64 URL
app.use('/upload', express.static(path.join(__dirname, 'uploads')));

// API endpoint to fetch images and their data
app.post('/images', async (req, res) => {
  const { usercustomerId } = req.body;
  console.log('Received usercustomerId:', usercustomerId);
  try {
    // Assuming you have a list of image names in an array called 'imageNames'
    const imageNames = ['images1.jpg', 'images2.jpg', 'images3.jpg', 'images4.jpg', 'images5.jpg'];

    // Create a list of image objects with image data (cover_images_id, name, status, date_added) and URL
    const imagesList = imageNames.map((imageName, index) => {
      const imageUrl = `/upload/passportImages/${imageName}`; // This will generate the URL for the image
      const imageId = (index + 1).toString(); // Generate sequential image IDs starting from 1

      return {
        cover_images_id: imageId,
        name: imageName,
        status: 'Active', // Set the status to 'Active' or any other value as required
        date_added: '', // You can set this to empty as the images are not in the database
        url: imageUrl,
      };
    });

    // Respond with the list of images and their data
    res.status(200).json({
      code: 200,
      status: 'success',
      data: imagesList,
    });
  } catch (error) {
    console.error('Error fetching images:', error);
    res.status(500).json({ status: 'error', error: 'Internal Server Error' });
  }
});



// Login Endpoint
app.post('/login', async (req, res) => {
    const { email, password } = req.body;

  
    try {
      const client = await pool.connect();
      const result = await client.query('SELECT * FROM users WHERE email = $1', [email]);
  
      if (result.rows.length === 0) {
        res.status(401).json({ code: 401, status: 'error', error: 'Invalid email or password' });
      } else {
        const user = result.rows[0];
        const match = await bcrypt.compare(password, user.password);
  
        if (match) {
          res.status(200).json({
            code: 200,
            status: 'success',
            data: {
              usercustomerId: user.id,
              email: user.email,
              // You can include other user data here if needed
            },
            
          });
        } else {
          res.status(401).json({ code: 401, status: 'error', error: 'Invalid email or password' });
        }
      }
  
      client.release();
    } catch (error) {
      console.error('Error during login:', error);
      res.status(500).json({ code: 500, status: 'error', error: 'Internal Server Error' });
    }
});
  

app.listen(port,'https://eigix.net/zain/api/', () => {
  console.log(`Server running on https://eigix.net/zain/api/:${port}`);
});

const server = app.listen(port, 'https://eigix.net/zain/api', () => {
  console.log(`Server is running on https://eigix.net/zain:${port}`);
});

// If there's an error, log the error message
server.on('error', (err) => {
  console.error('Server error:', err.message);
});